@@ -1,4 +1,4 @@
-/* Copyright (c) 2010, 2012, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2010, 2022, Oracle and/or its affiliates.
    Copyright (c) 2017, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify
@@ -31,7 +31,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 /* For my_chsize */
 #include <my_sys.h>
 /* For socket api */
-#ifdef __WIN__
+#ifdef _WIN32
   #include <ws2def.h>
   #include <winsock2.h>
   #include <MSWSock.h>
@@ -47,6 +47,10 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 
 #include "mysql/psi/psi.h"
 
+#ifndef PSI_SOCKET_CALL
+#define PSI_SOCKET_CALL(M) PSI_DYNAMIC_CALL(M)
+#endif
+
 /**
   @defgroup Socket_instrumentation Socket Instrumentation
   @ingroup Instrumentation_interface
@@ -65,11 +69,21 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
     do {} while (0)
 #endif
 
+/** An instrumented socket. */
 struct st_mysql_socket
 {
   /** The real socket descriptor. */
   my_socket fd;
 
+  /** Is this a Unix-domain socket? */
+  char is_unix_domain_socket;
+
+  /** Is this a socket opened for the extra port? */
+  char is_extra_port;
+
+  /** Address family of the socket. (See sa_family from struct sockaddr). */
+  unsigned short address_family;
+
   /**
     The instrumentation hook.
     Note that this hook is not conditionally defined,
@@ -100,16 +114,15 @@ typedef struct st_mysql_socket MYSQL_SOCKET;
 static inline MYSQL_SOCKET
 mysql_socket_invalid()
 {
-  MYSQL_SOCKET mysql_socket= {INVALID_SOCKET, NULL};
+  MYSQL_SOCKET mysql_socket= {INVALID_SOCKET, 0, 0, 0, NULL};
   return mysql_socket;
 }
 
 /**
   Set socket descriptor and address.
   @param socket nstrumented socket
-  @param fd socket descriptor
   @param addr unformatted socket address
-  @param adr_len length of socket address
+  @param addr_len length of socket address
 */
 
 static inline void
@@ -134,7 +147,6 @@ mysql_socket_set_address(
 /**
   Set socket descriptor and address.
   @param socket instrumented socket
-  @param thread instrumented owning thread
 */
 static inline void
 mysql_socket_set_thread_owner(
@@ -201,7 +213,6 @@ mysql_socket_setfd(MYSQL_SOCKET *mysql_socket, my_socket fd)
   @param STATE locker state
   @param SOCKET instrumented socket
   @param OP The socket operation to be performed
-  @param FLAGS per-socket operation flags.
   @param COUNT bytes to be written/read
   @sa MYSQL_END_SOCKET_WAIT.
 */
@@ -230,6 +241,13 @@ mysql_socket_setfd(MYSQL_SOCKET *mysql_socket, my_socket fd)
     do {} while (0)
 #endif
 
+/**
+  @def MYSQL_SOCKET_SET_STATE
+  Set the state (IDLE, ACTIVE) of an instrumented socket.
+  @param SOCKET the instrumented socket
+  @param STATE the new state
+  @sa PSI_socket_state
+*/
 #ifdef HAVE_PSI_SOCKET_INTERFACE
   #define MYSQL_SOCKET_SET_STATE(SOCKET, STATE) \
     inline_mysql_socket_set_state(SOCKET, STATE)
@@ -286,6 +304,22 @@ inline_mysql_socket_set_state(MYSQL_SOCKET socket, enum PSI_socket_state state)
 }
 #endif /* HAVE_PSI_SOCKET_INTERFACE */
 
+/**
+  @def mysql_socket_fd(K, F)
+  Create a socket.
+  @c mysql_socket_fd is a replacement for @c socket.
+  @param K PSI_socket_key for this instrumented socket
+  @param F File descriptor
+*/
+
+#ifdef HAVE_PSI_SOCKET_INTERFACE
+  #define mysql_socket_fd(K, F) \
+    inline_mysql_socket_fd(K, F)
+#else
+  #define mysql_socket_fd(K, F) \
+    inline_mysql_socket_fd(F)
+#endif
+
 /**
   @def mysql_socket_socket(K, D, T, P)
   Create a socket.
@@ -325,8 +359,8 @@ inline_mysql_socket_set_state(MYSQL_SOCKET socket, enum PSI_socket_state state)
   Return port number and IP address of the local host
   @c mysql_socket_getsockname is a replacement for @c getsockname.
   @param FD Instrumented socket descriptor returned by socket()
-  @param A  Pointer to returned address of local host in sockaddr structure
-  @param L  Pointer to length of sockaddr structure
+  @param AP  Pointer to returned address of local host in @c sockaddr structure
+  @param LP  Pointer to length of @c sockaddr structure
 */
 #ifdef HAVE_PSI_SOCKET_INTERFACE
   #define mysql_socket_getsockname(FD, AP, LP) \
@@ -430,7 +464,7 @@ inline_mysql_socket_set_state(MYSQL_SOCKET socket, enum PSI_socket_state state)
   @param N  Maximum bytes to receive
   @param FL Control flags
   @param AP Pointer to source address in sockaddr_storage structure
-  @param L  Size of sockaddr_storage structure
+  @param LP Size of sockaddr_storage structure
 */
 #ifdef HAVE_PSI_SOCKET_INTERFACE
   #define mysql_socket_recvfrom(FD, B, N, FL, AP, LP) \
@@ -476,6 +510,19 @@ inline_mysql_socket_set_state(MYSQL_SOCKET socket, enum PSI_socket_state state)
     inline_mysql_socket_setsockopt(FD, LV, ON, OP, OL)
 #endif
 
+/**
+  @def mysql_sock_set_nonblocking
+  Set socket to non-blocking.
+  @param FD instrumented socket descriptor
+*/
+#ifdef HAVE_PSI_SOCKET_INTERFACE
+  #define mysql_sock_set_nonblocking(FD) \
+    inline_mysql_sock_set_nonblocking(__FILE__, __LINE__, FD)
+#else
+  #define mysql_sock_set_nonblocking(FD) \
+    inline_mysql_sock_set_nonblocking(FD)
+#endif
+
 /**
   @def mysql_socket_listen(FD, N)
   Set socket state to listen for an incoming connection.
@@ -547,6 +594,39 @@ static inline void inline_mysql_socket_register(
 }
 #endif
 
+/** mysql_socket_fd */
+
+static inline MYSQL_SOCKET
+inline_mysql_socket_fd
+(
+#ifdef HAVE_PSI_SOCKET_INTERFACE
+  PSI_socket_key key,
+#endif
+  int fd)
+{
+  MYSQL_SOCKET mysql_socket= MYSQL_INVALID_SOCKET;
+  mysql_socket.fd= fd;
+
+  DBUG_ASSERT(mysql_socket.fd != INVALID_SOCKET);
+#ifdef HAVE_PSI_SOCKET_INTERFACE
+  mysql_socket.m_psi= PSI_SOCKET_CALL(init_socket)
+    (key, (const my_socket*)&mysql_socket.fd, NULL, 0);
+#endif
+  /**
+    Currently systemd socket activation is the user of this
+    function. Its API (man sd_listen_fds) says FD_CLOSE_EXEC
+    is already called. If there becomes another user, we
+    can call it again without detriment.
+
+    If needed later:
+    #if defined(HAVE_FCNTL) && defined(FD_CLOEXEC)
+        (void) fcntl(mysql_socket.fd, F_SETFD, FD_CLOEXEC);
+    #endif
+  */
+
+  return mysql_socket;
+}
+
 /** mysql_socket_socket */
 
 static inline MYSQL_SOCKET
@@ -972,6 +1052,78 @@ inline_mysql_socket_setsockopt
   return result;
 }
 
+/** set_socket_nonblock */
+static inline int
+set_socket_nonblock(my_socket fd)
+{
+  int ret= 0;
+#ifdef _WIN32
+  {
+    u_long nonblocking= 1;
+    ret= ioctlsocket(fd, FIONBIO, &nonblocking);
+  }
+#else
+  {
+    int fd_flags;
+    fd_flags= fcntl(fd, F_GETFL, 0);
+    if (fd_flags < 0)
+      return errno;
+#if defined(O_NONBLOCK)
+    fd_flags |= O_NONBLOCK;
+#elif defined(O_NDELAY)
+    fd_flags |= O_NDELAY;
+#elif defined(O_FNDELAY)
+    fd_flags |= O_FNDELAY;
+#else
+#error "No definition of non-blocking flag found."
+#endif /* O_NONBLOCK */
+    if (fcntl(fd, F_SETFL, fd_flags) == -1)
+      ret= errno;
+  }
+#endif /* _WIN32 */
+  return ret;
+}
+
+/** mysql_socket_set_nonblocking */
+
+static inline int
+inline_mysql_sock_set_nonblocking
+(
+#ifdef HAVE_PSI_SOCKET_INTERFACE
+  const char *src_file, uint src_line,
+#endif
+  MYSQL_SOCKET mysql_socket
+)
+{
+  int result= 0;
+
+#ifdef HAVE_PSI_SOCKET_INTERFACE
+  if (mysql_socket.m_psi)
+  {
+    /* Instrumentation start */
+    PSI_socket_locker *locker;
+    PSI_socket_locker_state state;
+    locker= PSI_SOCKET_CALL(start_socket_wait)
+        (&state, mysql_socket.m_psi, PSI_SOCKET_OPT,
+         (size_t)0, src_file, src_line);
+
+    /* Instrumented code */
+    result= set_socket_nonblock(mysql_socket.fd);
+
+    /* Instrumentation end */
+    if (locker != NULL)
+      PSI_SOCKET_CALL(end_socket_wait)(locker, (size_t)0);
+
+    return result;
+  }
+#endif
+
+  /* Non instrumented code */
+  result= set_socket_nonblock(mysql_socket.fd);
+
+  return result;
+}
+
 /** mysql_socket_listen */
 
 static inline int
@@ -1024,8 +1176,7 @@ inline_mysql_socket_accept
   int flags __attribute__ ((unused));
 #endif
 
-  MYSQL_SOCKET socket_accept= MYSQL_INVALID_SOCKET;
-  socklen_t addr_length= (addr_len != NULL) ? *addr_len : 0;
+  MYSQL_SOCKET socket_accept;
 
 #ifdef HAVE_PSI_SOCKET_INTERFACE
   if (socket_listen.m_psi != NULL)
@@ -1038,15 +1189,18 @@ inline_mysql_socket_accept
 
     /* Instrumented code */
 #ifdef HAVE_ACCEPT4
-    socket_accept.fd= accept4(socket_listen.fd, addr, &addr_length,
-                              SOCK_CLOEXEC);
+    socket_accept.fd= accept4(socket_listen.fd, addr, addr_len, SOCK_CLOEXEC);
 #else
-    socket_accept.fd= accept(socket_listen.fd, addr, &addr_length);
+    socket_accept.fd= accept(socket_listen.fd, addr, addr_len);
 #ifdef FD_CLOEXEC
-    flags= fcntl(socket_accept.fd, F_GETFD);
-    if (flags != -1) {
-      flags |= FD_CLOEXEC;
-      fcntl(socket_accept.fd, F_SETFD, flags);
+    if (socket_accept.fd != INVALID_SOCKET)
+    {
+      flags= fcntl(socket_accept.fd, F_GETFD);
+      if (flags != -1)
+      {
+        flags |= FD_CLOEXEC;
+        fcntl(socket_accept.fd, F_SETFD, flags);
+      }
     }
 #endif
 #endif
@@ -1060,15 +1214,18 @@ inline_mysql_socket_accept
   {
     /* Non instrumented code */
 #ifdef HAVE_ACCEPT4
-    socket_accept.fd= accept4(socket_listen.fd, addr, &addr_length,
-                              SOCK_CLOEXEC);
+    socket_accept.fd= accept4(socket_listen.fd, addr, addr_len, SOCK_CLOEXEC);
 #else
-    socket_accept.fd= accept(socket_listen.fd, addr, &addr_length);
+    socket_accept.fd= accept(socket_listen.fd, addr, addr_len);
 #ifdef FD_CLOEXEC
-    flags= fcntl(socket_accept.fd, F_GETFD);
-    if (flags != -1) {
-      flags |= FD_CLOEXEC;
-      fcntl(socket_accept.fd, F_SETFD, flags);
+    if (socket_accept.fd != INVALID_SOCKET)
+    {
+      flags= fcntl(socket_accept.fd, F_GETFD);
+      if (flags != -1)
+      {
+        flags |= FD_CLOEXEC;
+        fcntl(socket_accept.fd, F_SETFD, flags);
+      }
     }
 #endif
 #endif
@@ -1079,7 +1236,7 @@ inline_mysql_socket_accept
   {
     /* Initialize the instrument with the new socket descriptor and address */
     socket_accept.m_psi= PSI_SOCKET_CALL(init_socket)
-      (key, (const my_socket*)&socket_accept.fd, addr, addr_length);
+      (key, (const my_socket*)&socket_accept.fd, addr, *addr_len);
   }
 #endif
 
@@ -1139,7 +1296,7 @@ inline_mysql_socket_shutdown
 {
   int result;
 
-#ifdef __WIN__
+#ifdef _WIN32
   static LPFN_DISCONNECTEX DisconnectEx = NULL;
   if (DisconnectEx == NULL)
   {
@@ -1162,7 +1319,7 @@ inline_mysql_socket_shutdown
       (&state, mysql_socket.m_psi, PSI_SOCKET_SHUTDOWN, (size_t)0, src_file, src_line);
 
     /* Instrumented code */
-#ifdef __WIN__
+#ifdef _WIN32
     if (DisconnectEx)
       result= (DisconnectEx(mysql_socket.fd, (LPOVERLAPPED) NULL,
                             (DWORD) 0, (DWORD) 0) == TRUE) ? 0 : -1;
@@ -1179,7 +1336,7 @@ inline_mysql_socket_shutdown
 #endif
 
   /* Non instrumented code */
-#ifdef __WIN__
+#ifdef _WIN32
   if (DisconnectEx)
     result= (DisconnectEx(mysql_socket.fd, (LPOVERLAPPED) NULL,
                           (DWORD) 0, (DWORD) 0) == TRUE) ? 0 : -1;
