@@ -180,7 +180,7 @@ enum enum_indicator_type
 #define NUM_FLAG	32768U		/* Field is num (for clients) */
 #define PART_KEY_FLAG	16384U		/* Intern; Part of some key */
 #define GROUP_FLAG	32768U		/* Intern: Group field */
-#define BINCMP_FLAG	131072U		/* Intern: Used by sql_yacc */
+#define CONTEXT_COLLATION_FLAG 131072U  /* Intern: Used by sql_yacc */
 #define GET_FIXED_FIELDS_FLAG (1U << 18) /* Used to get fields in item tree */
 #define FIELD_IN_PART_FUNC_FLAG (1U << 19)/* Field part of partition func */
 #define PART_INDIRECT_KEY_FLAG (1U << 20)
@@ -197,13 +197,13 @@ enum enum_indicator_type
 #define FIELD_FLAGS_COLUMN_FORMAT_MASK (3U << FIELD_FLAGS_COLUMN_FORMAT)
 #define FIELD_IS_DROPPED (1U << 26)     /* Intern: Field is being dropped */
 
-#define VERS_SYS_START_FLAG (1 << 27)   /* autogenerated column declared with
+#define VERS_ROW_START (1 << 27)   /* autogenerated column declared with
                                              `generated always as row start`
                                               (see II.a SQL Standard) */
-#define VERS_SYS_END_FLAG (1 << 28)     /* autogenerated column declared with
+#define VERS_ROW_END (1 << 28)     /* autogenerated column declared with
                                            `generated always as row end`
                                             (see II.a SQL Standard).*/
-#define VERS_SYSTEM_FIELD (VERS_SYS_START_FLAG | VERS_SYS_END_FLAG)
+#define VERS_SYSTEM_FIELD (VERS_ROW_START | VERS_ROW_END)
 #define VERS_UPDATE_UNVERSIONED_FLAG (1 << 29) /* column that doesn't support
                                                 system versioning when table
                                                 itself supports it*/
