@@ -539,7 +539,8 @@ public:
   int32 get_sql_delay() { return sql_delay; }
   void set_sql_delay(int32 _sql_delay) { sql_delay= _sql_delay; }
   time_t get_sql_delay_end() { return sql_delay_end; }
-
+  rpl_gtid last_seen_gtid;
+  ulong last_trans_retry_count;
 private:
 
 
@@ -640,6 +641,33 @@ struct inuse_relaylog {
   }
 };
 
+enum start_alter_state
+{
+  INVALID= 0,
+  REGISTERED,           // Start Alter exist, Default state
+  COMMIT_ALTER,         // COMMIT the alter
+  ROLLBACK_ALTER,       // Rollback the alter
+  COMPLETED             // COMMIT/ROLLBACK Alter written in binlog
+};
+
+struct start_alter_info
+{
+  /*
+    ALTER id is defined as a pair of GTID's seq_no and domain_id.
+  */
+  decltype(rpl_gtid::seq_no) sa_seq_no; // key for searching (SA's id)
+  uint32 domain_id;
+  bool   direct_commit_alter; // when true CA thread executes the whole query
+  /*
+    0 prepared and not error from commit and rollback
+    >0 error expected in commit/rollback
+    Rollback can be logged with 0 error if master is killed
+  */
+  uint error;
+  enum start_alter_state state;
+  /* We are not using mysql_cond_t because we do not need PSI */
+  mysql_cond_t start_alter_cond;
+};
 
 /*
   This is data for various state needed to be kept for the processing of
@@ -667,6 +695,8 @@ struct rpl_group_info
   */
   uint64 gtid_sub_id;
   rpl_gtid current_gtid;
+  /* Currently applied event or NULL */
+  Log_event *current_event;
   uint64 commit_id;
   /*
     This is used to keep transaction commit order.
@@ -759,6 +789,9 @@ struct rpl_group_info
   bool did_mark_start_commit;
   /* Copy of flags2 from GTID event. */
   uchar gtid_ev_flags2;
+  /* Copy of flags3 from GTID event. */
+  uint16 gtid_ev_flags_extra;
+  uint64 gtid_ev_sa_seq_no;
   enum {
     GTID_DUPLICATE_NULL=0,
     GTID_DUPLICATE_IGNORE=1,
@@ -833,6 +866,15 @@ struct rpl_group_info
     RETRY_KILL_KILLED
   };
   uchar killed_for_retry;
+  bool reserved_start_alter_thread;
+  bool finish_event_group_called;
+  /*
+    Used for two phase alter table
+  */
+  rpl_parallel_thread *rpt;
+  Query_log_event *start_alter_ev;
+  bool direct_commit_alter;
+  start_alter_info *sa_info;
 
   rpl_group_info(Relay_log_info *rli_);
   ~rpl_group_info();
@@ -960,6 +1002,19 @@ struct rpl_group_info
     if (!is_parallel_exec)
       rli->event_relay_log_pos= future_event_relay_log_pos;
   }
+
+  void finish_start_alter_event_group();
+
+  bool get_finish_event_group_called()
+  {
+    return finish_event_group_called;
+  }
+
+  void set_finish_event_group_called(bool value)
+  {
+    finish_event_group_called= value;
+  }
+
 };
 
 
